
/*
 *
 * Copyright (c) 1998 Weyerhaeuser Co.
 *
 * Permission to use, copy, modify, distribute, and sell this software and 
 * its documentation for any purpose is hereby granted without fee, provided
 * that (i) the above copyright notices and this permission notice appear in
 * all copies of the software and related documentation, and (ii) the names of
 * David Swartz and Weyerhaeuser Co. may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of David Swartz and Weyerhaeuser Co.
 * 
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
 * 
 * IN NO EVENT SHALL DAVID SWARTZ OR WEYERHAEUSER COMPANY BE LIABLE FOR
 * ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
 * OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF 
 * LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
 * OF THIS SOFTWARE.
 */


/***************************************************************************
	VPPJava Version 1.0
	Author:	David Swartz	dswartz@wdni.com
	Date:	July 20, 1998

	Description:
	This DLL contains four native methods for acquiring images from the MRT Micro
	Video Port Professional PCMCIA frame grabber.  Java_ij_plugin_VPP_VPPInit() is
	used to initialize the frame grabber and must be called first.  
	Java_ij_plugin_VPP_VPPCaptureGray() and Java_ij_plugin_VPP_VPPCaptureRGB() are
	used to acquire either 8 bit Gray Scale images or 24 bit RGB images. 
	Java_ij_plugin_VPP_VPPClose() is used to shutdown the acquisition.  These 
	functions are called from a Java application such as VPPJava.java.  

	This DLL makes extensive use VPX32.dll which is avaliable as part of the 
	Software Developers Kit (http://www.mrtmicro.com/vpsdk302.zip).  Please get 
	this installed and running first.

	This DLL was compiled and tested using Microsoft Visual C version 5.0 running
	under Windows NT 4.0. 
	
	Note:	If you try to recompile this make sure that in Developer Studio your
	Project.Settings.C/C++.Preprocessor.AdditionalIncludeDirectories your include 
	directories match your environment instead of mine.

  ***************************************************************************/


#include <jni.h>
#include <stdio.h>
#include "ij_plugin_VPP.h"
#include "Vpptools.h"
#include "Vpx.h"

#define		COLOR_RGB	4		// from ImagePlus.java


/* Check return value from VPP functions */
void Check(VPP_Errno err)
{
	if (err != VPP_success)
	{
		fprintf(stderr,
			"Error: \"%s\" in function %s \n",
			VPP_errorString(err),
			VPP_lastErrorFunctionName());
			getchar();		// pause until user input
		exit(1);
	}
}


JNIEXPORT jboolean JNICALL Java_ij_plugin_VPP_VPPInit (JNIEnv *env, jobject obj, jint BitDepth, jint iWidth, jint iHeight)
{
	VPP_SnapData snapData;
	VPP_LimitData limitData;
	VPP_Byte value;
	
	if (BitDepth == 0)	{	// gray8

		VPP_VideoStandard videoStandard = VPP_NTSC;

		Check(VPP_init());
	//	Check(VPP_videoConfig( VPP_DEFAULT_CHANNEL, VPP_monochrome, &videoStandard ));
		Check(VPP_videoConfig( VPP_DEFAULT_CHANNEL, VPP_composite, &videoStandard ));

		snapData.monochrome = VPP_true;
	}

	else if (BitDepth == COLOR_RGB )	{	

		VPP_VideoStandard videoStandard = VPP_NTSC;
		Check(VPP_init());
		Check(VPP_videoConfig( VPP_DEFAULT_CHANNEL, VPP_composite, &videoStandard ));
		snapData.monochrome = VPP_false;
	}
	else	{
		fprintf(stderr, "Error: Unsupported bit depth passed to VPPInit.  ");
		return 0;
	}

	// The following code is undocumented in the MRT SDK
	// Many thanks to GF at MRT for the information on how 
	// to do this.

	// turn AGC on the framegrabber off
	Check( VPP_getDigitizerRegister(0x1A, &value));
	value = value | 0x40;
	Check( VPP_setDigitizerRegister(0x1A, value));
	// end of AGC code  

	Check( VPP_getLimits( VPP_NTSC, &limitData ));

	snapData.xPixels = (unsigned short)iWidth;

	if (snapData.xPixels > limitData.xActiveMax)	{
		fprintf(stderr, "Error: Image width must be less than %d\n",limitData.xActiveMax);
		return 0;
	}
	
	snapData.xOffset = (limitData.xActiveMax - snapData.xPixels) / 2;
	snapData.xActive = snapData.xPixels;
	snapData.yPixels = (unsigned short)iHeight;
	
	if (snapData.yPixels > limitData.yActiveMax)	{
		fprintf(stderr, "Error: Image height must be less than %d\n",limitData.yActiveMax);
		return 0;
	}

//	snapData.yOffset = (limitData.yActiveMax - snapData.yPixels) / 2;
	snapData.yOffset = 20;	// may introduce problems for large images 
	snapData.yActive = snapData.yPixels;

	snapData.interlaced = VPP_true;
	Check( VPP_prepareSnap( &snapData) );

	return 1;
}		

JNIEXPORT jboolean JNICALL Java_ij_plugin_VPP_VPPCaptureGray (JNIEnv *env, jobject obj, jbyteArray jbytePixels, jint iWidth, jint iHeight)
{
	int y;	// row counter

	// get local pointer to java array
	jsize len = (*env)->GetArrayLength(env, jbytePixels);
	jbyte *pixels = (*env)->GetByteArrayElements(env, jbytePixels, 0);
	jbyte *tempPixels = pixels;
		
	Check( VPP_startSnap() );
	Check( VPP_finishSnap() );
	Check( VPP_startReadout( VPP_mono, 0) );

	for( y=0; y< iHeight; y++)
	{
		Check( VPP_readLine( tempPixels, 0, (unsigned short)iWidth ));
		tempPixels += iWidth * sizeof(char);
	}

	(*env)->ReleaseByteArrayElements(env, jbytePixels, pixels, 0);
	
	return 1;
}

JNIEXPORT jboolean JNICALL Java_ij_plugin_VPP_VPPCaptureRGB (JNIEnv *env, jobject obj, jintArray jintPixels, jint iWidth, jint iHeight)
{

	int y;

	// get local pointer to java array
	jsize len = (*env)->GetArrayLength(env, jintPixels);
	jint *pixels = (*env)->GetIntArrayElements(env, jintPixels, 0);
	jint *tempPixels = pixels;
		
	Check( VPP_startSnap() );
	Check( VPP_finishSnap() );
	Check( VPP_startReadout( VPP_BGR032, 0) );

	for( y=0; y< iHeight; y++)
	{
		Check( VPP_readLine( tempPixels, 0, (unsigned short)iWidth));
		tempPixels += iWidth;
	}

	// The BGR032 format from the VPP sets all the alpha values to 0.  
	// we need the value to be 255 in order to see the image.  
	
	tempPixels = pixels;
	for(y=0; y<len; y++)	{
		*tempPixels= *tempPixels + 0xFF000000;
		tempPixels++;
	}

	(*env)->ReleaseIntArrayElements(env, jintPixels, pixels, 0);
	
	return 1;
}

JNIEXPORT jboolean JNICALL Java_ij_plugin_VPP_VPPClose (JNIEnv *env, jobject obj)
{
	VPP_closedown( 0 );
	return 1;
}
