/*
Copyright (c) 1998 Weyerhaeuser Co.

Permission to use, copy, modify, distribute, and sell this software and 
its documentation for any purpose is hereby granted without fee, provided
that (i) the above copyright notices and this permission notice appear in
all copies of the software and related documentation, and (ii) the names of
David Swartz and Weyerhaeuser Co. may not be used in any advertising or
publicity relating to the software without the specific, prior written
permission of David Swartz and Weyerhaeuser Co.
 
THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
 
IN NO EVENT SHALL DAVID SWARTZ OR WEYERHAEUSER COMPANY BE LIABLE FOR
ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF 
LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
OF THIS SOFTWARE.
*/
package ij.plugin;
import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import java.io.*;
import ij.*;
import ij.io.*;
import ij.gui.ImageCanvas;
import ij.process.*;

/**
 *  <P>
 * Image/J plug-in for acquiring images from the MRT Video Port Professional 
 * @version    1.1, November 9, 1998
 * @author	    David Swartz
 *
*/

/* revision history
   July 8, 1998		Version 1.0		Image/J 0.80
   Nov. 9, 1998		Version 1.1		Image/J 0.93	
	- Rewrote VPPOpener because of problems with FileOpener based version and
	 Java's fallback to the default RGB Color model and the int type when byte was expected.
	 This caused the ByteProcessor to crash.  
*/

public class VPP implements PlugIn {

    private native boolean VPPInit( int BitDepth, int iWidth, int iHeight);  
    private native boolean VPPCaptureGray( byte[] bytePixels, int iWidth, int iHeight );
    private native boolean VPPCaptureRGB( int[] intPixels, int iWidth, int iHeight );
    private native boolean VPPClose();	 
    
    static  {
	System.loadLibrary("VPPJava");	    // The file VPPJava.dll must be avaliable to use this plugin.
										// If using jdb then this file must be renamed VPPJava_g.dll
	}
		
    public void run(String arg) {
    
	/* 
	To enable live aquisition the following changes need to be made to ImageCanvas.java 
	1.  Line 23 add -> public static boolean bLive;  // Added by David Swartz for VPP plugin
	2.  Line 433 change -> public void mouseClicked(MouseEvent e) {}
	    To -> public void mouseClicked(MouseEvent e) { bLive = false; }  // Changed by David Swartz for VPP plugin 
	
	The program will acquire the image continually until the user gives a single mouse click.
	If you don't want to make this change then comment out the following statement and the section 
	noted later in this file.
    	*/
    	ImageCanvas.bLive = true;	// Set's a flag that goes false when the mouse is clicked.
	
    	ImageJ ij = Info.getImageJ();
    	FileInfo fi = new FileInfo();	
    	ImagePlus imp = WindowManager.getCurrentImage();
    	VPPOpener fo;
		
	if (imp == null)    {	    // Create an image
		
	    // Set image parameters
	    fi.fileFormat = fi.UNKNOWN;	    // "Unknown" will keep revert from doing anything
	    fi.fileType = fi.GRAY8;	    // default image type when creating a new image
	    //fi.fileType = fi.RGB;			// uncomment out this line if you want the default to be RGB
	    fi.fileName = "Camera";
	    fi.directory = "";
	    fi.width = 640;
	    fi.height = 480;
	    fi.offset = 0;
	
	    fo = new VPPOpener(fi, ij);
	    imp = WindowManager.getCurrentImage();
	    if (imp == null) return;	// image still not created so exit
	}
	else	{	// the image all ready exists
 
	    switch(imp.getType())   {	    //since inconsistent use of constants for file/image types ...
	    	case imp.GRAY8:	
	    	case imp.COLOR_RGB:
			case imp.RGBA:				// added for a test
				if(Info.debugMode) Info.write( "Image format is" + String.valueOf(imp.getType()));
				break;
	    	default:
	    	    Info.write("Unsupported file type");
	    	    return;	
	    }     
	}

	if(VPPInit( imp.getType(), imp.getWidth(), imp.getHeight()))	{  // Initialize VPP
	    update(imp);    // grab at least one image  
		    
	    /* Comment out the section after this line if you don't want to modify ImageCanvas.java 
	    as described at the top of this file.  This will prevent live aquisition.*/
		    
	    while( ImageCanvas.bLive ) {
	    	update(imp);
	    	Info.showStatus("Camera Live");
	    }
	    /* end of commented out section */	
	    // For our application we automatically convert RGB images to grayscale
	    // You may not need this for your application
//		if (imp.getType() == 4) convertToGrayscale(imp); // for RGB images
	}
	VPPClose();	
	Info.showStatus("Camera Closed");
    }
	
	
    // This method updates the image with new image data 
    public void update(ImagePlus imp)	{
	ImageProcessor ip = imp.getProcessor();
	    
	if (imp==null)	{ Info.noImage(); return; }
	    
	if (imp.getType() == imp.GRAY8)   {
	    byte[] bytePixels = (byte[]) ip.getPixels();
	    VPPCaptureGray( bytePixels, imp.getWidth(), imp.getHeight() );
	}
	    
	if (imp.getType() ==  imp.COLOR_RGB)  {
	    int[] intPixels = (int[]) ip.getPixels();
	    VPPCaptureRGB( intPixels, imp.getWidth(), imp.getHeight() );
    	    }
		
	imp.updateAndDraw();
    }
  
    // This is an exact copy of the function in ij.Converter
    // Yuk, there has to be a way to call it directly, let's put that on the back burner to figure out.
    static void convertToGrayscale(ImagePlus imp) {
		int c, r, g, b;
		int width, height;
		int[] pixels32;
		byte[] pixels8;
		ColorModel cm;
		Image img8;
		ImageProcessor ip;
		
		//get RGB pixels
		Info.showProgress(0.1);
		width = imp.getWidth();
		height = imp.getHeight();
		if (imp.getType()==imp.COLOR_RGB)
			ip = imp.getProcessor();
		else
			ip = new ColorProcessor(imp.getImage());
		pixels32 = (int[])ip.getPixels();
		imp.killProcessor();
		
		//convert to grayscale
		pixels8 = new byte[width * height];
		for (int i=0; i < width*height; i++) {
			c = pixels32[i];
			r = (c&0xff0000)>>16;
			g = (c&0xff00)>>8;
			b = c&0xff;
			pixels8[i] = (byte)((int)(r*0.30 + g*0.59 + b*0.11) & 0xff);
			if (i%10000==0)
				Info.showProgress(0.1 + 0.9*((double)i/(width*height)));
		}

		//create an 8-bit grayscale image
		cm = LookUpTable.makeGrayscaleColorModel(false);
	        img8 = Toolkit.getDefaultToolkit().createImage(new MemoryImageSource(width, height, cm, pixels8, 0, width));
		Undo.setup(Undo.TYPE_CONVERSION, imp);
		imp.setImage(img8);
		imp.draw();
	}

}   // end of class VPP


// This is a stripped down version of ij.gui.NewImage that opens up a blank image to the screen.

class VPPOpener {

	public static final int GRAY8 = 0;
	public static final int GRAY16 = 1;
	public static final int GRAY32 = 4;
	public static final int RGB = 6;

	public static final int FILL_WHITE = 0;
	public static final int FILL_BLACK = 1;
	public static final int FILL_RAMP = 2;

    private FileInfo fi;
    private ImageJ ij;
    private int width, height, fileType;
	private String fileName;
	private static int nSlices = 1;
    
	public VPPOpener(FileInfo fi, ImageJ ij)	{	
		this.fi = fi;
    	this.ij = ij;
		width = fi.width;
		height = fi.height;
		fileType = fi.fileType;
		fileName = fi.fileName;
		open(fileName, width, height, 1, fi.fileType, FILL_WHITE);
    }	
    
	public VPPOpener( ImageJ ij)	{
		this.ij = ij;
	}
  
	public static ImagePlus createByteImage(String title, int width, int height, int slices, int fill) {
		byte[] pixels = new byte[width*height];
		switch (fill) {
			case FILL_WHITE:
				for (int i=0; i<width*height; i++)
					pixels[i] = (byte)255;
				break;
			case FILL_BLACK:
				break;
			case FILL_RAMP:
				for (int y=0; y<height; y++)
					for (int x=0; x<width; x++)
						pixels[y*width+x] = (byte)((x*255)/(width-1));
				break;
		}
		ColorModel cm = LookUpTable.makeGrayscaleColorModel(false);
		Image img = Toolkit.getDefaultToolkit().createImage(new MemoryImageSource(width, height, cm, pixels, 0, width));
		ImagePlus imp = new ImagePlus(title, img, Info.getImageJ());
		if (slices>1) {
			ImageStack stack = imp.getStack();
			try {
				stack.addSlice("", pixels);
				for (int i=2; i<=nSlices; i++) {
					byte[] pixels2 = new byte[width*height];
					System.arraycopy(pixels, 0, pixels2, 0, width*height);
					stack.addSlice("", pixels2);
				}
			}
			catch(OutOfMemoryError e) {
				Info.outOfMemory(title);
				stack.deleteLastSlice();
				stack.deleteLastSlice();
			}
		if (stack.getSize()<=1)
			imp.killStack();
		}
		return imp;
	}

	public static ImagePlus createRGBImage(String title, int width, int height, int fill) {
		int[] pixels = new int[width*height];
		switch (fill) {
			case FILL_WHITE:
				for (int i=0; i<width*height; i++)
					pixels[i] = -1;
				break;
			case FILL_BLACK:
				for (int i=0; i<width*height; i++)
					pixels[i] = 0xff000000;
				break;
			case FILL_RAMP:
				int r,g,b;
				for (int y=0; y<height; y++)
					for (int x=0; x<width; x++) {
						r = g = b = (byte)((x*255)/(width-1));
						pixels[y*width+x] = 0xff000000 + (r<<16) + (g<<8) + b;
				}
				break;
		}
		Image img = Toolkit.getDefaultToolkit().createImage(new MemoryImageSource(width, height, pixels, 0, width));
		return(new ImagePlus(title, img, Info.getImageJ()));
	}

	public static ImagePlus createShortImage(String title, int width, int height, int fill) {
		short[] pixels = new short[width*height];
		switch (fill) {
			case FILL_WHITE: case FILL_BLACK:
				break;
			case FILL_RAMP:
				for (int y=0; y<height; y++)
					for (int x=0; x<width; x++)
						pixels[y*width+x] = (short)((x*2000)/(width-1)-1000);
				break;
		}
		return(new ImagePlus(title, pixels, width, height, Info.getImageJ()));
	}

	public static ImagePlus createFloatImage(String title, int width, int height, int fill) {
		float[] pixels = new float[width*height];
		switch (fill) {
			case FILL_WHITE: case FILL_BLACK:
				break;
			case FILL_RAMP:
				for (int y=0; y<height; y++)
					for (int x=0; x<width; x++)
						pixels[y*width+x] = (float)((x*1.0)/(width-1));
				break;
		}
		return(new ImagePlus(title, pixels, width, height, Info.getImageJ()));
	}

	public static void open(String title, int width, int height, int nSlices, int type, int fill) {
		ImagePlus imp = null;
		switch (type) {
			case GRAY8:
				imp = createByteImage(title, width, height, nSlices, fill);
				break;
			case GRAY16:
				imp = createShortImage(title, width, height, fill);
				break;
			case GRAY32:
				imp = createFloatImage(title, width, height, fill);
				break;
			case RGB:
				imp = createRGBImage(title, width, height, fill);
				break;
		}
		if (imp!=null)
			imp.show();
	}

}   // end of class VPPOpener
    

    